module AlarmClock(	Clock,		// 27MHz Reference Clock
			Reset,		// Power-up reset

			SetHour,	// The set-hour button
			SetMinute,	// The set-minute button

			AlarmMode,	// 1 if the toggle is set to "alarm mode"
			AlarmOn,	// 1 if the alarm is activated

			Snooze,		// The snooze button

			DisplayHour,	// The hour to display
			DisplayMinute,	// the minute to display
			AlarmSounding	// 1 if the alarm is sounding
		);

	input		Clock, Reset;
	input		SetHour, SetMinute;
	input		AlarmMode, AlarmOn;
	input		Snooze;

	output	[4:0]	DisplayHour;
	output	[5:0]	DisplayMinute;
	output		AlarmSounding;

	wire		Minute, Hour;
	wire		NextClockMinute, NextClockHour;
	wire		NextAlarmMinute, NextAlarmHour;
	wire		SnoozeDone;

	wire	[4:0]	ClockHour, AlarmHour;
	wire	[5:0]	ClockMinute, AlarmMinute;
	
	wire		AlarmStart;
	reg		AlarmPending;

	assign	DisplayHour =		AlarmMode ? AlarmHour : ClockHour;
	assign	DisplayMinute =		AlarmMode ? AlarmMinute : ClockMinute;

	assign	NextClockMinute =	Minute | (~AlarmMode & SetMinute);
	assign	NextClockHour =		(Hour & Minute & ~SetMinute) | (~AlarmMode & SetHour);
	assign	NextAlarmMinute =	AlarmMode & SetMinute;
	assign	NextAlarmHour =		AlarmMode & SetHour;

	assign	AlarmStart =		AlarmOn & (AlarmHour == ClockHour) & (AlarmMinute == ClockMinute);
	assign	AlarmSounding =		(AlarmPending | AlarmStart) & SnoozeDone;

	always @ (posedge Clock) begin
		if (Reset | (Minute & AlarmSounding)) AlarmPending <= 1'b0;
		else if (AlarmStart) AlarmPending <= 1'b1;
	end

	Divide2Minute	D2M_Inst(	.Clock(		Clock),
					.Reset(		Reset),
					.Minute(	Minute));

	MinuteCount	MC_Clock(	.Clock(		Clock),
					.Reset(		Reset),
					.Enable(	NextClockMinute),
					.Minute(	ClockMinute),
					.Rollover(	Hour));

	HourCount	HC_Clock(	.Clock(		Clock),
					.Reset(		Reset),
					.Enable(	NextClockHour),
					.Hour(		ClockHour),
					.Rollover(	));

	MinuteCount	MC_Alarm(	.Clock(		Clock),
					.Reset(		Reset),
					.Enable(	NextAlarmMinute),
					.Minute(	AlarmMinute),
					.Rollover(	));

	HourCount	HC_Alarm(	.Clock(		Clock),
					.Reset(		Reset),
					.Enable(	NextAlarmHour),
					.Hour(		AlarmHour),
					.Rollover(	));

	SnoozeDelay	SD_Inst(	.Clock(		Clock),
					.Reset(		Reset),
					.Set(		Snooze & AlarmSounding),
					.Enable(	Minute),
					.Done(		SnoozeDone));
endmodule